from typing import Optional, List

from pydantic import BaseModel


class Context(BaseModel):
    send_type: str = 'assistant'
    sender_type: str = "assistant"
    text: str = ''


class RoleMeta(BaseModel):
    user_name: Optional[str] = 'user'
    bot_name: Optional[str] = 'assistant'


class OpenaiChatStreamBody(BaseModel):
    """
    parameter follows: https://platform.openai.com/docs/api-reference/chat/create
    """
    model: Optional[str] = 'gpt-3.5-turbo'
    prompt: str
    temperature: Optional[float] = 1
    messages: Optional[List[Context]] = None
    role_meta: Optional[RoleMeta] = None
    type: Optional[str] = 'text'
    timeout: Optional[int] = 600



import logging


def log(name):
    """
    @param name: python file name
    @return: Logger
    """
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter(
        "%(levelname)s:     %(asctime)s - %(module)s-%(funcName)s-line:%(lineno)d - %(message)s"
    )
    ch = logging.StreamHandler()
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger


def clear_other_log():
    for name, item in logging.Logger.manager.loggerDict.items():
        if not isinstance(item, logging.Logger):
            continue
        if "aoe" not in name:
            item.setLevel(logging.CRITICAL)


clear_other_log()
logger = log("util")



from fastapi import APIRouter

from openaoe.backend.model.xunfei import XunfeiSparkChatBody
from openaoe.backend.service.service_xunfei import spark_chat_svc

router = APIRouter()


@router.post("/v1/spark/chat", tags=["Spark"])
async def spark_chat(body: XunfeiSparkChatBody):
    """
    chat api for Xunfei Spark model
    @param body: request body
    @return: response
    """
    ret = spark_chat_svc(body)
    return ret



from fastapi import APIRouter, Request

from openaoe.backend.model.openai import OpenaiChatStreamBody
from openaoe.backend.service.service_openai import chat_completion_stream

router = APIRouter()


@router.post("/v1/text/chat-stream", tags=["OpenAI"])
async def openai_chat_stream(request: Request, body: OpenaiChatStreamBody):
    """
    stream api for OpenAI ChatCompletion
    @param request: fastapi request
    @param body: request body
    @return: response
    """
    ret = chat_completion_stream(request, body)
    return ret



import base64
import hashlib
import hmac
import json
import urllib
from urllib.parse import urlencode

from websocket import create_connection

from openaoe.backend.config.biz_config import get_model_configuration, get_base_url
from openaoe.backend.config.constant import PROVIDER_XUNFEI
from openaoe.backend.model.aoe_response import AOEResponse
from openaoe.backend.model.xunfei import XunfeiSparkChatBody
from openaoe.backend.util.log import log
from openaoe.backend.util.time_util import get_current_date

logger = log(__name__)


def _calc_authorization(ak: str, sk: str, date: str, host: str) -> str:
    msg = "host: " + host + "\n"
    msg += "date: " + date + "\n"
    msg += "GET " + "/v2.1/chat" + " HTTP/1.1"
    msg_sha = hmac.new(sk.encode('utf-8'), msg.encode('utf-8'), digestmod=hashlib.sha256).digest()
    signature = base64.b64encode(msg_sha).decode(encoding='utf-8')
    authorization_origin = f'api_key="{ak}", algorithm="hmac-sha256", headers="host date request-line", signature="{signature}"'
    authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')
    return authorization


def _websocket_process(url: str, body: dict):
    ws = create_connection(url)
    try:
        ws.send(json.dumps(body))
    except Exception as e:
        logger.error(e)

    res = ""
    while True:
        rcv_text = ws.recv()
        content = json.loads(rcv_text)
        header = content.get("header")
        if header.get("code") != 0:
            res = "error"
            # spark failed
            logger.error(f"spark failed: {content}")
            break
        payload = content.get("payload")
        text = payload.get("choices").get("text")
        for item in text:
            res += item.get("content")
        if header.get("status") == 2:
            break
    ws.close()
    return res


def spark_chat_svc(body: XunfeiSparkChatBody):
    """
    chat logic for spark model
    """
    api_base = get_base_url(PROVIDER_XUNFEI)
    app_id = get_model_configuration(PROVIDER_XUNFEI, "app_id")
    ak = get_model_configuration(PROVIDER_XUNFEI, "ak")
    sk = get_model_configuration(PROVIDER_XUNFEI, "sk")

    url_parse = urllib.parse.urlparse(api_base)
    host = url_parse.hostname

    date = get_current_date()
    authorization = _calc_authorization(ak=ak, sk=sk, date=date, host=host)
    v = {
        "authorization": authorization,
        "date": date,
        "host": host
    }
    v_urlencode = urlencode(v)
    url = f"{api_base}/v2.1/chat?{v_urlencode}"
    texts = [
        {"role": item.role, "content": item.content}
        for item in body.payload.message.text or []
    ]
    uid = None
    if body.header is not None:
        uid = None if body.header.uid is None else body.header.uid
    body = {
        "header": {
            "app_id": app_id,
            "uid": uid
        },
        "parameter": {
            "chat": {
                "domain": body.parameter.chat.domain,
                "temperature": body.parameter.chat.temperature,
                "max_tokens": body.parameter.chat.max_tokens
            }
        },
        "payload": {
            "message": {
                "text": texts
            }
        }
    }
    try:
        r = _websocket_process(url, body)
        return AOEResponse(data=r)
    except Exception as e:
        logger.error(e)
        return AOEResponse(
            msg="error",
            msgCode="-1",
            data=str(e)
        )



from pydantic import BaseModel
from typing import Optional, List


class Message(BaseModel):
    content: str
    author: Optional[str] = 0


class Prompt(BaseModel):
    messages: List[Message]


class GooglePalmChatBody(BaseModel):
    """
    parameter follows: https://ai.google.dev/api/rest/v1beta/models/generateMessage
    """
    model: Optional[str] = "chat-bison-001"
    prompt: Prompt
    temperature: Optional[float] = 0.1
    candidate_count: Optional[int] = 1






from jsonstreamer import ObjectStreamer
from sse_starlette import EventSourceResponse

from openaoe.backend.config.biz_config import get_api_key, get_base_url
from openaoe.backend.config.constant import *
from openaoe.backend.model.aoe_response import AOEResponse
from openaoe.backend.model.google import GooglePalmChatBody
from openaoe.backend.service.base import base_request, base_stream
from openaoe.backend.util.log import log

logger = log(__name__)


async def palm_chat_svc(body: GooglePalmChatBody):
    """
    chat logic for google PaLM model
    """
    url, params, request_body = _construct_request_data(body)

    if "gemini" in body.model:
        return EventSourceResponse(
            base_stream(PROVIDER_GOOGLE, url, "post", {}, _catch_all, body=request_body, params=params))

    try:
        response = await base_request(PROVIDER_GOOGLE, url, "post", {}, request_body, params=params)
        response_json = response.data
        if response_json.get('error') is not None:
            err_msg = response_json.get('error').get("message")
            return AOEResponse(
                msg="error",
                msgCode="-1",
                data=err_msg
            )

        # remove messages
        if response_json and response_json.get("messages"):
            response_json.pop("messages")
        base = AOEResponse(
            data=response_json
        )
    except Exception as e:
        logger.error(f"{e}")
        base = AOEResponse(
            msg="error",
            msgCode="-1",
            data=str(e)
        )
    return base


def _construct_request_data(body: GooglePalmChatBody):
    model = body.model
    api_base = get_base_url(PROVIDER_GOOGLE, body.model)
    api_key = get_api_key(PROVIDER_GOOGLE, body.model)

    if "gemini" in model:
        # specially process gemini request
        url = f"{api_base}/v1beta/models/{model}:streamGenerateContent"
        params = {
            "key": api_key
        }

        if body.prompt.messages[0].author == "1":
            body.prompt.messages = body.prompt.messages[1:]
        contents = [
            {
                "role": "user" if msg.author == "0" else "model",
                "parts": [{
                    "text": msg.content
                }]
            }
            for msg in body.prompt.messages
        ]
        body = {
            "contents": contents,
            "generationConfig": {
                "temperature": body.temperature,
                "candidateCount": 1
            }
        }
    else:
        url = f"{api_base}/v1beta2/models/{model}:generateMessage?key={api_key}"
        params = {
            "key": api_key
        }

        messages = []
        last_author = ""
        # ignore not answered prompt
        for item in body.prompt.messages:
            if item.author == last_author:
                messages.pop()
            messages.append({"content": item.content, "author": item.author})
            last_author = item.author
        body = {
            "prompt": {
                "messages": messages
            },
            "temperature": body.temperature,
            "candidate_count": body.candidate_count,
            "model": body.model
        }
    return url, params, body


def _catch_all(event_name, *args):
    if event_name == ObjectStreamer.PAIR_EVENT and args[0] == "text":
        print(args[1])
        return

    elif event_name != ObjectStreamer.ELEMENT_EVENT:
        return

    for item in args:
        try:
            text = item["candidates"][0]["content"]["parts"][0]["text"]
            print(text)
        except:
            logger.warning(f"parse error, raw: {item}")



import json

import requests
from fastapi.encoders import jsonable_encoder
from sse_starlette.sse import EventSourceResponse

from openaoe.backend.config.biz_config import get_base_url
from openaoe.backend.config.constant import *
from openaoe.backend.model.aoe_response import AOEResponse
from openaoe.backend.model.internlm import InternlmChatCompletionBody
from openaoe.backend.util.log import log

logger = log(__name__)


def chat_completion_v1(request, body: InternlmChatCompletionBody):
    messages = body.messages
    msgs = []
    for msg in messages:
        if msg.sender_type == "bot":
            msg.sender_type = body.role_meta.bot_name
        msg_item = {
            "role": msg.sender_type,
            "content": jsonable_encoder(msg.text),
        }
        msgs.append(msg_item)
    msg_item = {
        "role": "user",
        "content": body.prompt
    }
    msgs.append(msg_item)
    # restful api
    url = get_base_url(PROVIDER_INTERNLM, body.model) + "/v1/chat/completions"
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    data = {
        "model": body.model,
        "messages": msgs,
        "temperature": body.temperature,
        "top_p": body.top_p,
        "n": body.n,
        "max_tokens": body.max_tokens,
        "stream": body.stream,
        "presence_penalty": body.presence_penalty,
        "frequency_penalty": body.frequency_penalty,
        "user": "string",
        "repetition_penalty": 1,
        "session_id": -1,
        "ignore_eos": False
    }
    if body.stream:
        return chat_completion_stream_v1(request, url, headers, data)
    else:
        res = requests.post(url, headers=headers, json=data)
        if res and res.status_code == 200:
            return AOEResponse(
                data=res.json()
            )
        else:
            return AOEResponse(
                data="request failed"
            )


def chat_completion_stream_v1(request, url, headers, data):
    # retry times
    count = 3

    async def event_generator_json(count: int):
        while count > 0:
            count -= 1
            stop_flag = False
            if await request.is_disconnected():
                break
            try:
                res = requests.post(url, headers=headers, json=data, stream=True)
                logger.debug(f"url={url}, headers={headers}, body={data}")
                if res:
                    for chunk in res.iter_content(chunk_size=512, decode_unicode=True):
                        logger.debug(f"chunk: {chunk}")
                        res_data = chunk.replace("data: ", "")
                        try:
                            json_data = json.loads(res_data)
                            logger.debug(f"json_data: {json_data}")
                            if json_data.get("object") == "error":
                                error_msg = json_data.get("message")
                                dict_item = {
                                    "success": "true",
                                    "msg": f"LMDeploy: {error_msg} "
                                }
                                yield json.dumps(dict_item, ensure_ascii=False)
                                stop_flag = True
                            choices = json_data.get("choices")
                            if not choices:
                                continue
                            choice = choices[0]
                            if choice.get('finish_reason'):
                                stop_flag = True
                            if 'content' not in choice['delta']:
                                continue
                            s = choice["delta"]["content"]
                            logger.debug(f"content={s}")
                            if s:
                                dict_item = {
                                    "success": "true",
                                    "msg": s
                                }
                                yield json.dumps(dict_item, ensure_ascii=False)
                            if stop_flag:
                                break
                        except Exception as e:
                            logger.error(f"Error: {e}")
                    if stop_flag:
                        break
            except Exception as e:
                logger.error(f"{e}")
                yield json.dumps({
                    "success": "false",
                    "msg": f"from backend: {str(e)}"
                })
                break

    return EventSourceResponse(event_generator_json(count))



#!/usr/bin/env python3
import json
import sys
import traceback
from io import StringIO
from typing import Callable

import httpx
from fastapi.encoders import jsonable_encoder
from jsonstreamer import ObjectStreamer

from openaoe.backend.config.constant import DEFAULT_TIMEOUT_SECONDS
from openaoe.backend.model.aoe_response import AOEResponse, StreamResponse
from openaoe.backend.util.log import log

logger = log(__name__)


async def base_request(provider: str, url: str, method: str, headers: dict, body=None, timeout=DEFAULT_TIMEOUT_SECONDS,
                       params=None,
                       files=None) -> AOEResponse:
    """
    common request function for http
    Args:
        provider: use for log
        url: complete url
        method: request method
        headers: request headers, excluding user-agent, host and ip.
        body: json body only
        timeout: seconds
        params: request params
        files: request file

    Returns:
        AOEResponse
    """
    response = AOEResponse()

    headers_pure = {}
    for k, v in headers:
        k = k.lower()
        if k == "user-agent" or k == "host" or "ip" in k:
            continue
        headers_pure[k] = v

    body_str = body
    if "content-type" in headers and "multipart/form-data" in headers["content-type"]:
        body_str = "image"
    if len(body_str) > 200:
        body_str = body_str[:200]

    try:
        async with httpx.AsyncClient() as client:
            proxy = await client.request(method, url, headers=headers, json=body, timeout=timeout,
                                         params=params, files=files)
        response.data = proxy.content
        try:
            response.data = json.loads(response.data)
        except:
            response.data = proxy.content

    except Exception as e:
        response.msg = str(e)
        logger.error(f"[{provider}] url: {url}, method: {method}, headers: {jsonable_encoder(headers)}, "
                     f"body: {body_str} failed, response: {jsonable_encoder(response)}")
    return response


async def base_stream(provider: str, url: str, method: str, headers: dict, stream_callback: Callable, body=None,
                      timeout=DEFAULT_TIMEOUT_SECONDS,
                      params=None,
                      files=None):
    """
    common stream request
    Args:
        stream_callback:
        provider: use for log
        url: complete url
        method: request method
        headers: request headers, excluding user-agent, host and ip.
        stream_callback: use ObjectStream to stream parse json, this method will be executed while any stream received,
                         use print to output(we have redirected stdout to response stream)
        body: json body only
        timeout: seconds
        params: request params
        files: request file

    Returns:
        SSE response with StreamResponse json string
    """
    headers_pure = {
        "Content-Type": "application/json"
    }
    for k, v in headers:
        k = k.lower()
        if k == "user-agent" or k == "host" or "ip" in k:
            continue
        headers_pure[k] = v

    body_str = jsonable_encoder(body)
    if "content-type" in headers and "multipart/form-data" in headers["content-type"]:
        body_str = "image"
    if len(body_str) > 200:
        body_str = body_str[:200]

    try:
        with httpx.stream(method, url, json=body, params=params, files=files, headers=headers_pure,
                          timeout=timeout) as res:
            if res.status_code != 200:
                raise Exception(f"request failed, model status code: {res.status_code}")

            # stream parser
            streamer = ObjectStreamer()
            sys.stdout = mystdout = StringIO()
            streamer.add_catch_all_listener(stream_callback)

            for text in res.iter_text():
                streamer.consume(text)
                res = mystdout.getvalue()
                stream_res = json.dumps(jsonable_encoder(StreamResponse(msg=res)))
                # format res
                yield stream_res
                # clear printed string
                sys.stdout.seek(0)
                sys.stdout.truncate()

    except Exception as e:
        print(traceback.format_exc())
        res = json.dumps(jsonable_encoder(StreamResponse(
            success=False,
            msg=str(e)
        )))
        logger.error(f"[{provider}] url: {url}, method: {method}, headers: {jsonable_encoder(headers_pure)}, "
                     f"body: {body_str} failed, response: {res}")
        yield res



from fastapi import APIRouter, Request

from openaoe.backend.model.internlm import InternlmChatCompletionBody
from openaoe.backend.service.service_internlm import chat_completion_v1

router = APIRouter()


@router.post("/v1/chat/completions", tags=["Internlm"])
async def internlm_chat_completions_v1(request: Request, body: InternlmChatCompletionBody):
    """
    Internlm ChatCompletion api
    @param request: fastapi request
    @param body: body
    @return: response
    """
    ret = chat_completion_v1(request, body)
    return ret



TYPE_BOT = "bot"
TYPE_USER = "user"
TYPE_SYSTEM = "system"

DATE_PATTERN = "%Y%m%d"
TIMEOUT_SECONDS = 30

PROVIDER_OPENAI = "openai"
PROVIDER_MINIMAX = "minimax"
PROVIDER_GOOGLE = "google"
PROVIDER_XUNFEI = "spark"
PROVIDER_CLAUDE = "claude"
PROVIDER_INTERNLM = "internlm"

DEFAULT_TIMEOUT_SECONDS = 600




import os

import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import FileResponse
from starlette.responses import HTMLResponse

from openaoe.backend.api.route_claude import router as claude
from openaoe.backend.api.route_google import router as google
from openaoe.backend.api.route_internlm import router as internlm
from openaoe.backend.api.route_minimax import router as minimax
from openaoe.backend.api.route_openai import router as openai
from openaoe.backend.api.route_xunfei import router as xunfei
from openaoe.backend.config.biz_config import img_out_path, init_config
from openaoe.backend.util.log import log
from openaoe.backend.util.str_util import safe_join


logger = log(__name__)
# define global variable
API_VER = 'v1'
base_dir = os.path.dirname(os.path.abspath(__file__))
STATIC_RESOURCE_DIR = os.path.join(base_dir, "frontend", "dist")
ASSETS_RESOURCE_DIR = os.path.join(STATIC_RESOURCE_DIR, "assets")

path = img_out_path()

# init configuration content
BIZ_CONFIG = init_config()

app = FastAPI()


@app.get("/config/json")
async def get_config_json():
    return BIZ_CONFIG.json


@app.get("/", response_class=HTMLResponse)
@app.get("/home", response_class=HTMLResponse)
async def server():
    return FileResponse(f"{STATIC_RESOURCE_DIR}/index.html")


@app.get("/assets/{path:path}")
async def build_resource(path: str):
    print(ASSETS_RESOURCE_DIR, path)
    static_file = safe_join(ASSETS_RESOURCE_DIR, path)
    print(static_file)
    return FileResponse(static_file)


@app.get("/{path:path}")
async def build_resource(path: str):
    static_file = safe_join(STATIC_RESOURCE_DIR, path)
    return FileResponse(static_file)


# add middlewares here if you need
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# add api routers
app.include_router(minimax, prefix=f"/{API_VER}/minimax")
app.include_router(openai, prefix=f"/{API_VER}/openai")
app.include_router(google, prefix=f"/{API_VER}/google")
app.include_router(claude, prefix=f"/{API_VER}/claude")
app.include_router(xunfei, prefix=f"/{API_VER}/xunfei")
app.include_router(internlm, prefix=f"/{API_VER}/internlm")


def main():
    """
    main function
    start server use uvicorn, default workers: 3
    """
    uvicorn.run(
        "openaoe.main:app",
        host='0.0.0.0',
        port=10099,
        timeout_keep_alive=600,
        workers=3
    )


if __name__ == "__main__":
    main()



from fastapi import APIRouter, Request

from openaoe.backend.model.minimax import MinimaxChatCompletionBody
from openaoe.backend.service.service_minimax import minimax_chat_stream_svc

router = APIRouter()


@router.post("/v1/text/chat-stream", tags=["MiniMax"])
async def minimax_chat_stream(request: Request, body: MinimaxChatCompletionBody):
    """
    chat stream api for Minimax
    @param request: fastapi request
    @param body: request body
    @return: response
    """
    body.stream = True
    ret = minimax_chat_stream_svc(request, body)
    return ret



from typing import Optional, List

from pydantic import BaseModel


class Header(BaseModel):
    app_id: Optional[str] = None
    uid: Optional[str] = None


class Chat(BaseModel):
    domain: str = "generalv2"
    temperature: Optional[float] = 0.5
    max_tokens: Optional[int] = 2048
    top_k: Optional[int] = 4
    chat_id: Optional[str]


class Parameter(BaseModel):
    chat: Chat


class Text(BaseModel):
    role: str
    content: str


class Message(BaseModel):
    text: List[Text]


class Payload(BaseModel):
    message: Message


class XunfeiSparkChatBody(BaseModel):
    """
    parameter follows: https://www.xfyun.cn/doc/spark/Web.html#_1-%E6%8E%A5%E5%8F%A3%E8%AF%B4%E6%98%8E
    """
    header: Optional[Header] = None
    parameter: Parameter
    payload: Payload



from typing import Optional, List, Union

from openai._types import NOT_GIVEN
from pydantic import BaseModel


class Context(BaseModel):
    sender_type: str = "assistant"
    text: str = ''


class RoleMeta(BaseModel):
    user_name: Optional[str] = 'user'
    bot_name: Optional[str] = 'assistant'


class InternlmChatCompletionBody(BaseModel):
    model: Optional[str] = 'internlm-chat-7b'
    role_meta: Optional[RoleMeta]
    prompt: Optional[str]
    messages: List[Context]
    temperature: Optional[float] = 0.7
    top_p: Optional[float] = 0.8
    n: Optional[int] = 1
    stop: Optional[Union[str, List]] = "false"
    presence_penalty: Optional[float] = 0
    frequency_penalty: Optional[float] = 0.5
    user: Optional[str] = NOT_GIVEN
    stream: Optional[bool] = False
    timeout: Optional[int] = 600
    session_id: Optional[int] = -1
    ignore_eos: Optional[bool] = False
    max_tokens: Optional[int] = 1024



from datetime import datetime
from time import mktime
from wsgiref.handlers import format_date_time


def get_current_date() -> str:
    cur_time = datetime.now()
    date = format_date_time(mktime(cur_time.timetuple()))
    return date



#!/usr/bin/env python3

import argparse
import os
import sys
from copy import deepcopy

import yaml

from openaoe.backend.util.log import log

logger = log(__name__)
BIZ_CONFIG = None


class BizConfig:
    def __init__(self, **args):
        # raw dict
        self.__dict__.update(args)

    # provider -> model_name -> ModelConfig
    @property
    def models_map(self):
        if "models" not in self.__dict__:
            return {}

        models_dict = self.__dict__["models"]
        ret = {}
        for model_name, model_config in models_dict.items():
            if model_config["provider"] not in ret:
                ret[model_config["provider"]] = {}
            ret[model_config["provider"]][model_name] = ModelConfig(model_config["webui"], model_config["api"])
        return ret

    @models_map.getter
    def __get_models_map(self):
        return self.models_map

    @property
    def json(self):
        ret = deepcopy(self.__dict__)
        if "models" in ret:
            models_config = ret["models"]
        for model_name, config in models_config.items():
            config.pop("api")
            ret["models"][model_name] = config
        return ret


class ModelConfig:
    def __init__(self, webui_config, api_config):
        self.webui_config = webui_config
        self.api_config = api_config


def init_config() -> BizConfig:
    parser = argparse.ArgumentParser(description="LLM group chat framework")
    parser.add_argument('-f', '--file', type=str, required=True, help='Path to the YAML config file.')
    config_path = parser.parse_args()
    logger.info(f"your config file is: {config_path.file}")
    return load_config(config_path.file)


def load_config(config_path) -> BizConfig:
    logger.info(f"start to init configuration from {config_path}.")
    if not os.path.isfile(config_path):
        logger.error(f"invalid path: {config_path}, not exist or not file")
        sys.exit(-1)

    with open(config_path) as fin:
        m = yaml.safe_load(fin)
        if not m or len(m) == 0:
            logger.error("init configuration failed. Exit")
            sys.exit(-1)

        global BIZ_CONFIG
        BIZ_CONFIG = BizConfig(**m)
    logger.info("init configuration successfully.")
    return BIZ_CONFIG


def get_model_configuration(provider: str, field, model_name: str = None):
    models = BIZ_CONFIG.models_map
    if not models:
        logger.error(f"invalid configuration file")
        sys.exit(-1)

    provider_config = models.get(provider)
    if provider_config:
        if model_name:
            try:
                return provider_config.get(model_name).api_config.get(field)
            except:
                for config_model_name, config in provider_config.items():
                    if config_model_name.startswith(model_name):
                        return config.api_config.get(field)
                logger.error(f"{provider} get field: {field} for model: {model_name} failed")
                return ""
        elif not model_name:
            # default the first provider ModelConfig
            provider_models_config_list = list(provider_config.values())
            try:
                logger.info(f"{provider} get field: {field} for anonymous model, use the first one as default.")
                return provider_models_config_list[0].api_config.get(field)
            except:
                logger.error(f"{provider} get field: {field} for anonymous model failed")
                return ""

    logger.error(f"provider: {provider} has no field: {field} configuration for model: {model_name}")
    return ""


def get_base_url(provider: str, model_name: str = None) -> str:
    return get_model_configuration(provider, "api_base", model_name)


def get_api_key(provider: str, model_name: str = None) -> str:
    return get_model_configuration(provider, "api_key", model_name)


def app_abs_path():
    return os.path.dirname(os.path.abspath(__file__)).split("/backend")[0]


def img_out_path():
    abs_path = app_abs_path()
    return f'{abs_path}/frontend/dist/tmp/img/out'


def img_in_path():
    abs_path = app_abs_path()
    return f'{abs_path}/frontend/dist/tmp/img/in'



from pydantic import BaseModel
from typing import Optional, List


class ClaudeMessage(BaseModel):
    # fixed string: user, bot, system
    role: str
    content: str


class ClaudeChatBody(BaseModel):
    """
    parameter follows: https://github.com/anthropics/anthropic-sdk-python
    model choose: https://docs.anthropic.com/claude/reference/selecting-a-model
    """
    model: Optional[str] = "claude-instant-1"
    max_tokens: Optional[int] = 300
    temperature: Optional[float] = 1.0
    messages: Optional[List[ClaudeMessage]] = None



import json
from typing import List

from anthropic import Anthropic, HUMAN_PROMPT, AI_PROMPT
from sse_starlette.sse import EventSourceResponse

from openaoe.backend.config.biz_config import get_api_key, get_base_url
from openaoe.backend.config.constant import TYPE_BOT, TYPE_USER, TYPE_SYSTEM
from openaoe.backend.config.constant import PROVIDER_CLAUDE
from openaoe.backend.model.aoe_response import AOEResponse
from openaoe.backend.model.claude import ClaudeChatBody, ClaudeMessage


def claude_chat_stream_svc(request, body: ClaudeChatBody):
    """
    stream api logic for Claude model
    use anthropic SDK: https://github.com/anthropics/anthropic-sdk-python
    """
    api_key = get_api_key(PROVIDER_CLAUDE, body.model)
    api_base = get_base_url(PROVIDER_CLAUDE, body.model)
    prompt = _gen_prompt(body.messages)
    if not prompt or len(prompt) == 0:
        return AOEResponse(
            msg="error",
            msgCode="-1",
            data="prompt or messages must be set"
        )

    anthropic = Anthropic(api_key=api_key, base_url=api_base)

    async def stream():
        try:
            conn = anthropic.completions.create(
                prompt=prompt,
                max_tokens_to_sample=body.max_tokens,
                model=body.model,
                stream=True,
            )
            stop_flag = False
            while True:
                if await request.is_disconnected():
                    break

                for msg in conn:
                    dict_item = {
                        "msg": "",
                        "success": "true"
                    }

                    if msg.stop_reason:
                        dict_item["stop_reason"] = msg.stop_reason
                        stop_flag = True

                    if msg.completion:
                        dict_item["success"] = "true"
                        dict_item["msg"] = msg.completion

                    if dict_item:
                        yield json.dumps(dict_item, ensure_ascii=False)
                if stop_flag:
                    break
            return

        except Exception as e:
            dict_item = {
                "success": "false",
                "msg": str(e)
            }
            yield json.dumps(dict_item)
            return

    return EventSourceResponse(stream())


def _gen_prompt(messages: List[ClaudeMessage]):
    ret = ""
    if not messages:
        return ret

    for item in messages:
        if item.role == TYPE_BOT:
            ret += f"{AI_PROMPT} {item.content} "
        elif item.role == TYPE_USER:
            ret += f"{HUMAN_PROMPT} {item.content} "
        elif item.role == TYPE_SYSTEM:
            ret = item.content + ret
    ret += f"{AI_PROMPT}"
    return ret



#!/usr/bin/env python3

from typing import Optional

from pydantic import BaseModel


class AOEResponse(BaseModel):
    """
    Standard OpenAOE response
    """
    msg: Optional[str] = "ok"
    msgCode: Optional[str] = "10000"
    data: Optional[object] = None


class StreamResponse(BaseModel):
    """
    Standard OpenAOE stream response
    """
    success: Optional[bool] = True
    msg: Optional[str] = ""



from fastapi import HTTPException
import os
from typing import List
import random
import string


def safe_join(directory: str, path: str) -> str:
    """
    Safely path to a base directory to avoid escaping the base directory.
    Borrowed from: werkzeug.security.safe_join
    @param directory:
    @param path:
    """
    _os_alt_seps: List[str] = [
        sep for sep in [os.path.sep, os.path.altsep] if sep is not None and sep != "/"
    ]

    if path == "":
        raise HTTPException(status_code=400, detail="path is empty")

    filename = os.path.normpath(path)
    full_path = os.path.join(directory, filename)
    if (
            any(sep in filename for sep in _os_alt_seps)
            or os.path.isabs(filename)
            or filename == ".."
            or filename.startswith("../")
            or os.path.isdir(full_path)
    ):
        raise HTTPException(status_code=400, detail="path is illegal")

    if not os.path.exists(full_path):
        raise HTTPException(status_code=404, detail="path is not existed")
    return full_path


def generate_random_string(length: int):
    letters = string.ascii_lowercase
    result_str = ''.join(random.choice(letters) for i in range(length))
    return result_str




from typing import Optional, List, Union

from pydantic import BaseModel


class RoleMeta(BaseModel):
    user_name: str
    bot_name: str


class Messages(BaseModel):
    sender_type: str
    text: str


class MinimaxChatCompletionBody(BaseModel):
    """
    parameter follows: https://api.minimax.chat/document/guides/chat-model/chat/api?id=6569c88e48bc7b684b3037a9#3.1%20%E8%AF%B7%E6%B1%82%E4%BD%93(request)%E5%8F%82%E6%95%B0
    """
    model: Optional[str] = 'abab5-chat'
    prompt: str
    role_meta: Optional[Union[RoleMeta, None]] = None
    messages: Optional[List[Messages]] = None
    stream: Optional[bool] = False
    type: Optional[str] = "text"
